@@ -43,7 +43,7 @@ class FocusedWave(AcousticField):
             print(f"Error generating file name: {e}")
             return None
 
-    def _apply_delay(self, dx=None):
+    def _apply_delay(self):
         """
         Apply a temporal delay to focus the wave at a given lateral position (x_focal) and fixed focal depth (Foc).
         Returns:
@@ -51,16 +51,13 @@ class FocusedWave(AcousticField):
         """
         try:
             # 1. Positions latérales de tous les éléments (en mètres)
-            Xmin, Xmax = self.params['Xrange'][0], self.params['Xrange'][1]
-            element_positions = np.linspace(Xmin, Xmax, self.params['num_elements'])
+            element_positions = np.linspace(self.params['Xrange'][0], self.params['Xrange'][1], self.params['num_elements'])
 
             # 2. Trouver l'indice de l'élément le plus proche de self.focal_line
             center_idx = np.argmin(np.abs(element_positions - self.focal_line))
 
-            # 3. Limites des indices pour N_piezoFocal éléments centrés sur self.focal_line
-            N_focal = self.params['N_piezoFocal']
-            start_idx = max(0, center_idx - N_focal // 2)
-            end_idx = min(self.params['num_elements'] - 1, start_idx + N_focal - 1)
+            start_idx = max(0, center_idx - self.params['N_piezoFocal'] // 2)
+            end_idx = min(self.params['num_elements'] - 1, start_idx + self.params['N_piezoFocal'] - 1)
 
             # 4. Positions des éléments sélectionnés (en mètres)
             selected_indices = np.arange(start_idx, end_idx + 1)
